@@ -9,43 +9,43 @@ use std::{
 };
 
 use fs_err as fs;
-use humansize::{format_size, DECIMAL};
 use same_file::Handle;
+use ubyte::ToByteUnit;
 
 use crate::{
     error::FinalError,
     info,
     list::FileInArchive,
-    utils::{self, FileVisibilityPolicy},
+    utils::{self, EscapedPathDisplay, FileVisibilityPolicy},
     warning,
 };
 
 /// Unpacks the archive given by `archive` into the folder given by `into`.
 /// Assumes that output_folder is empty
-pub fn unpack_archive(reader: Box<dyn Read>, output_folder: &Path) -> crate::Result<Vec<PathBuf>> {
+pub fn unpack_archive(reader: Box<dyn Read>, output_folder: &Path, quiet: bool) -> crate::Result<usize> {
     assert!(output_folder.read_dir().expect("dir exists").count() == 0);
     let mut archive = tar::Archive::new(reader);
 
-    let mut files_unpacked = vec![];
+    let mut files_unpacked = 0;
     for file in archive.entries()? {
         let mut file = file?;
 
-        let file_path = output_folder.join(file.path()?);
         file.unpack_in(output_folder)?;
 
         // This is printed for every file in the archive and has little
         // importance for most users, but would generate lots of
         // spoken text for users using screen readers, braille displays
         // and so on
+        if !quiet {
+            info!(
+                inaccessible,
+                "{:?} extracted. ({})",
+                utils::strip_cur_dir(&output_folder.join(file.path()?)),
+                file.size().bytes(),
+            );
 
-        info!(
-            inaccessible,
-            "{:?} extracted. ({})",
-            utils::strip_cur_dir(&output_folder.join(file.path()?)),
-            format_size(file.size(), DECIMAL),
-        );
-
-        files_unpacked.push(file_path);
+            files_unpacked += 1;
+        }
     }
 
     Ok(files_unpacked)
@@ -86,6 +86,7 @@ pub fn build_archive_from_paths<W>(
     output_path: &Path,
     writer: W,
     file_visibility_policy: FileVisibilityPolicy,
+    quiet: bool,
 ) -> crate::Result<W>
 where
     W: Write,
@@ -118,7 +119,9 @@ where
             // little importance for most users, but would generate lots of
             // spoken text for users using screen readers, braille displays
             // and so on
-            info!(inaccessible, "Compressing '{}'.", utils::to_utf(path));
+            if !quiet {
+                info!(inaccessible, "Compressing '{}'.", EscapedPathDisplay::new(path));
+            }
 
             if path.is_dir() {
                 builder.append_dir(path, path)?;
@@ -137,7 +140,7 @@ where
                 builder.append_file(path, file.file_mut()).map_err(|err| {
                     FinalError::with_title("Could not create archive")
                         .detail("Unexpected error while trying to read file")
-                        .detail(format!("Error: {}.", err))
+                        .detail(format!("Error: {err}."))
                 })?;
             }
         }
