@@ -13,6 +13,7 @@ use std::{
 use filetime::{set_file_mtime, FileTime};
 use fs_err as fs;
 use same_file::Handle;
+use time::OffsetDateTime;
 use ubyte::ToByteUnit;
 use zip::{self, read::ZipFile, DateTime, ZipArchive};
 
@@ -116,10 +117,7 @@ where
                     Err(e) => return Some(Err(e.into())),
                 };
 
-                let path = match file.enclosed_name() {
-                    Some(path) => path.to_owned(),
-                    None => return None,
-                };
+                let path = file.enclosed_name()?.to_owned();
                 let is_dir = file.is_dir();
 
                 Some(Ok(FileInArchive { path, is_dir }))
@@ -145,7 +143,9 @@ where
     W: Write + Seek,
 {
     let mut writer = zip::ZipWriter::new(writer);
-    let options = zip::write::FileOptions::default();
+    // always use ZIP64 to allow compression of files larger than 4GB
+    // the format is widely supported and the extra 20B is negligible in most cases
+    let options = zip::write::FileOptions::default().large_file(true);
     let output_handle = Handle::from_path(output_path);
 
     #[cfg(not(unix))]
@@ -219,7 +219,7 @@ where
                     options
                 };
 
-                let mut file = fs::File::open(entry.path())?;
+                let mut file = fs::File::open(path)?;
                 writer.start_file(
                     path.to_str().unwrap(),
                     options.last_modified_time(get_last_modified_time(&file)),
@@ -255,20 +255,20 @@ fn display_zip_comment_if_exists(file: &ZipFile) {
 fn get_last_modified_time(file: &fs::File) -> DateTime {
     file.metadata()
         .and_then(|metadata| metadata.modified())
-        .map_err(|_| ())
-        .and_then(|time| DateTime::from_time(time.into()))
+        .ok()
+        .and_then(|time| DateTime::try_from(OffsetDateTime::from(time)).ok())
         .unwrap_or_default()
 }
 
 fn set_last_modified_time(zip_file: &ZipFile, path: &Path) -> crate::Result<()> {
-    let modification_time_in_seconds = zip_file
-        .last_modified()
-        .to_time()
-        .expect("Zip archive contains a file with broken 'last modified time'")
-        .unix_timestamp();
+    let modification_time = zip_file.last_modified().to_time();
+
+    let Ok(time_in_seconds) = modification_time else {
+        return Ok(());
+    };
 
     // Zip does not support nanoseconds, so we can assume zero here
-    let modification_time = FileTime::from_unix_time(modification_time_in_seconds, 0);
+    let modification_time = FileTime::from_unix_time(time_in_seconds.unix_timestamp(), 0);
 
     set_file_mtime(path, modification_time)?;
 
