@@ -8,11 +8,7 @@ use fs_err as fs;
 use crate::{
     archive,
     commands::warn_user_about_loading_zip_in_memory,
-    extension::{
-        split_first_compression_format,
-        CompressionFormat::{self, *},
-        Extension,
-    },
+    extension::{split_first_compression_format, CompressionFormat::*, Extension},
     utils::{user_wants_to_continue, FileVisibilityPolicy},
     QuestionAction, QuestionPolicy, BUFFER_CAPACITY,
 };
@@ -27,6 +23,7 @@ use crate::{
 /// # Return value
 /// - Returns `Ok(true)` if compressed all files normally.
 /// - Returns `Ok(false)` if user opted to abort compression mid-way.
+#[allow(clippy::too_many_arguments)]
 pub fn compress_files(
     files: Vec<PathBuf>,
     extensions: Vec<Extension>,
@@ -35,25 +32,56 @@ pub fn compress_files(
     quiet: bool,
     question_policy: QuestionPolicy,
     file_visibility_policy: FileVisibilityPolicy,
+    level: Option<i16>,
 ) -> crate::Result<bool> {
     // If the input files contain a directory, then the total size will be underestimated
     let file_writer = BufWriter::with_capacity(BUFFER_CAPACITY, output_file);
 
-    let mut writer: Box<dyn Write> = Box::new(file_writer);
+    let mut writer: Box<dyn Send + Write> = Box::new(file_writer);
 
     // Grab previous encoder and wrap it inside of a new one
-    let chain_writer_encoder = |format: &CompressionFormat, encoder: Box<dyn Write>| -> crate::Result<Box<dyn Write>> {
-        let encoder: Box<dyn Write> = match format {
-            Gzip => Box::new(flate2::write::GzEncoder::new(encoder, Default::default())),
-            Bzip => Box::new(bzip2::write::BzEncoder::new(encoder, Default::default())),
-            Lz4 => Box::new(lzzzz::lz4f::WriteCompressor::new(encoder, Default::default())?),
-            Lzma => Box::new(xz2::write::XzEncoder::new(encoder, 6)),
-            Snappy => Box::new(snap::write::FrameEncoder::new(encoder)),
+    let chain_writer_encoder = |format: &_, encoder| -> crate::Result<_> {
+        let encoder: Box<dyn Send + Write> = match format {
+            Gzip => Box::new(
+                // by default, ParCompress uses a default compression level of 3
+                // instead of the regular default that flate2 uses
+                gzp::par::compress::ParCompress::<gzp::deflate::Gzip>::builder()
+                    .compression_level(
+                        level.map_or_else(Default::default, |l| gzp::Compression::new((l as u32).clamp(0, 9))),
+                    )
+                    .from_writer(encoder),
+            ),
+            Bzip => Box::new(bzip2::write::BzEncoder::new(
+                encoder,
+                level.map_or_else(Default::default, |l| bzip2::Compression::new((l as u32).clamp(1, 9))),
+            )),
+            Lz4 => Box::new(lzzzz::lz4f::WriteCompressor::new(
+                encoder,
+                lzzzz::lz4f::PreferencesBuilder::new()
+                    .compression_level(level.map_or(1, |l| (l as i32).clamp(1, lzzzz::lz4f::CLEVEL_MAX)))
+                    .build(),
+            )?),
+            Lzma => Box::new(xz2::write::XzEncoder::new(
+                encoder,
+                level.map_or(6, |l| (l as u32).clamp(0, 9)),
+            )),
+            Snappy => Box::new(
+                gzp::par::compress::ParCompress::<gzp::snap::Snap>::builder()
+                    .compression_level(gzp::par::compress::Compression::new(
+                        level.map_or_else(Default::default, |l| (l as u32).clamp(0, 9)),
+                    ))
+                    .from_writer(encoder),
+            ),
             Zstd => {
-                let zstd_encoder = zstd::stream::write::Encoder::new(encoder, Default::default());
+                let zstd_encoder = zstd::stream::write::Encoder::new(
+                    encoder,
+                    level.map_or(zstd::DEFAULT_COMPRESSION_LEVEL, |l| {
+                        (l as i32).clamp(zstd::zstd_safe::min_c_level(), zstd::zstd_safe::max_c_level())
+                    }),
+                );
                 // Safety:
-                //     Encoder::new() can only fail if `level` is invalid, but Default::default()
-                //     is guaranteed to be valid
+                //     Encoder::new() can only fail if `level` is invalid, but the level
+                //     is `clamp`ed and therefore guaranteed to be valid
                 Box::new(zstd_encoder.unwrap().auto_finish())
             }
             Tar | Zip => unreachable!(),
