@@ -1,11 +1,11 @@
 //! Our representation of all the supported compression formats.
 
-use std::{fmt, path::Path};
+use std::{ffi::OsStr, fmt, path::Path};
 
 use bstr::ByteSlice;
 
 use self::CompressionFormat::*;
-use crate::warning;
+use crate::{error::Error, warning};
 
 /// A wrapper around `CompressionFormat` that allows combinations like `tgz`
 #[derive(Debug, Clone, Eq)]
@@ -14,8 +14,9 @@ pub struct Extension {
     /// One extension like "tgz" can be made of multiple CompressionFormats ([Tar, Gz])
     pub compression_formats: &'static [CompressionFormat],
     /// The input text for this extension, like "tgz", "tar" or "xz"
-    pub display_text: String,
+    display_text: String,
 }
+
 // The display_text should be ignored when comparing extensions
 impl PartialEq for Extension {
     fn eq(&self, other: &Self) -> bool {
@@ -70,7 +71,7 @@ pub enum CompressionFormat {
 
 impl CompressionFormat {
     /// Currently supported archive formats are .tar (and aliases to it) and .zip
-    pub fn is_archive_format(&self) -> bool {
+    fn is_archive_format(&self) -> bool {
         // Keep this match like that without a wildcard `_` so we don't forget to update it
         match self {
             Tar | Zip => true,
@@ -84,29 +85,12 @@ impl CompressionFormat {
     }
 }
 
-impl fmt::Display for CompressionFormat {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let text = match self {
-            Gzip => ".gz",
-            Bzip => ".bz",
-            Zstd => ".zst",
-            Lz4 => ".lz4",
-            Lzma => ".lz",
-            Snappy => ".sz",
-            Tar => ".tar",
-            Zip => ".zip",
-        };
-
-        write!(f, "{text}")
-    }
-}
-
 pub const SUPPORTED_EXTENSIONS: &[&str] = &[
     "tar", "tgz", "tbz", "tlz4", "txz", "tzlma", "tsz", "tzst", "zip", "bz", "bz2", "gz", "lz4", "xz", "lzma", "sz",
     "zst",
 ];
 
-pub fn to_extension(ext: &[u8]) -> Option<Extension> {
+fn to_extension(ext: &[u8]) -> Option<Extension> {
     Some(Extension::new(
         match ext {
             b"tar" => &[Tar],
@@ -129,15 +113,32 @@ pub fn to_extension(ext: &[u8]) -> Option<Extension> {
     ))
 }
 
-pub fn split_extension<'a>(name: &mut &'a [u8]) -> Option<&'a [u8]> {
+fn split_extension(name: &mut &[u8]) -> Option<Extension> {
     let (new_name, ext) = name.rsplit_once_str(b".")?;
     if matches!(new_name, b"" | b"." | b"..") {
         return None;
     }
+    let ext = to_extension(ext)?;
     *name = new_name;
     Some(ext)
 }
 
+pub fn parse_format(fmt: &OsStr) -> crate::Result<Vec<Extension>> {
+    let fmt = <[u8] as ByteSlice>::from_os_str(fmt).ok_or_else(|| Error::InvalidFormat {
+        reason: "Invalid UTF-8".into(),
+    })?;
+
+    let mut extensions = Vec::new();
+    for extension in fmt.split_str(b".") {
+        let extension = to_extension(extension).ok_or_else(|| Error::InvalidFormat {
+            reason: format!("Unsupported extension: {}", extension.to_str_lossy()),
+        })?;
+        extensions.push(extension);
+    }
+
+    Ok(extensions)
+}
+
 /// Extracts extensions from a path.
 ///
 /// Returns both the remaining path and the list of extension objects
@@ -149,7 +150,7 @@ pub fn separate_known_extensions_from_name(path: &Path) -> (&Path, Vec<Extension
     };
 
     // While there is known extensions at the tail, grab them
-    while let Some(extension) = split_extension(&mut name).and_then(to_extension) {
+    while let Some(extension) = split_extension(&mut name) {
         extensions.insert(0, extension);
     }
 
@@ -169,22 +170,6 @@ pub fn extensions_from_path(path: &Path) -> Vec<Extension> {
     extensions
 }
 
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn test_extensions_from_path() {
-        use CompressionFormat::*;
-        let path = Path::new("bolovo.tar.gz");
-
-        let extensions: Vec<Extension> = extensions_from_path(path);
-        let formats: Vec<CompressionFormat> = flatten_compression_formats(&extensions);
-
-        assert_eq!(formats, vec![Tar, Gzip]);
-    }
-}
-
 // Panics if formats has an empty list of compression formats
 pub fn split_first_compression_format(formats: &[Extension]) -> (CompressionFormat, Vec<CompressionFormat>) {
     let mut extensions: Vec<CompressionFormat> = flatten_compression_formats(formats);
@@ -199,3 +184,76 @@ pub fn flatten_compression_formats(extensions: &[Extension]) -> Vec<CompressionF
         .copied()
         .collect()
 }
+
+/// Builds a suggested output file in scenarios where the user tried to compress
+/// a folder into a non-archive compression format, for error message purposes
+///
+/// E.g.: `build_suggestion("file.bz.xz", ".tar")` results in `Some("file.tar.bz.xz")`
+pub fn build_archive_file_suggestion(path: &Path, suggested_extension: &str) -> Option<String> {
+    let path = path.to_string_lossy();
+    let mut rest = &*path;
+    let mut position_to_insert = 0;
+
+    // Walk through the path to find the first supported compression extension
+    while let Some(pos) = rest.find('.') {
+        // Use just the text located after the dot we found
+        rest = &rest[pos + 1..];
+        position_to_insert += pos + 1;
+
+        // If the string contains more chained extensions, clip to the immediate one
+        let maybe_extension = {
+            let idx = rest.find('.').unwrap_or(rest.len());
+            &rest[..idx]
+        };
+
+        // If the extension we got is a supported extension, generate the suggestion
+        // at the position we found
+        if SUPPORTED_EXTENSIONS.contains(&maybe_extension) {
+            let mut path = path.to_string();
+            path.insert_str(position_to_insert - 1, suggested_extension);
+
+            return Some(path);
+        }
+    }
+
+    None
+}
+
+#[cfg(test)]
+mod tests {
+    use std::path::Path;
+
+    use super::*;
+
+    #[test]
+    fn test_extensions_from_path() {
+        use CompressionFormat::*;
+        let path = Path::new("bolovo.tar.gz");
+
+        let extensions: Vec<Extension> = extensions_from_path(path);
+        let formats: Vec<CompressionFormat> = flatten_compression_formats(&extensions);
+
+        assert_eq!(formats, vec![Tar, Gzip]);
+    }
+
+    #[test]
+    fn builds_suggestion_correctly() {
+        assert_eq!(build_archive_file_suggestion(Path::new("linux.png"), ".tar"), None);
+        assert_eq!(
+            build_archive_file_suggestion(Path::new("linux.xz.gz.zst"), ".tar").unwrap(),
+            "linux.tar.xz.gz.zst"
+        );
+        assert_eq!(
+            build_archive_file_suggestion(Path::new("linux.pkg.xz.gz.zst"), ".tar").unwrap(),
+            "linux.pkg.tar.xz.gz.zst"
+        );
+        assert_eq!(
+            build_archive_file_suggestion(Path::new("linux.pkg.zst"), ".tar").unwrap(),
+            "linux.pkg.tar.zst"
+        );
+        assert_eq!(
+            build_archive_file_suggestion(Path::new("linux.pkg.info.zst"), ".tar").unwrap(),
+            "linux.pkg.info.tar.zst"
+        );
+    }
+}
